@@ -1,7 +1,8 @@
+# -*- coding: utf-8 -*-
 '''
-    VERIDIC Project - Towards a centralized access control system
+    Authentic 2 - Versatile Identity Server
 
-    Copyright (C) 2011  Mikael Ates
+    Copyright (C) 2011 Entr'ouvert
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -22,31 +23,102 @@ import datetime
 import logging
 
 from cPickle import loads, dumps
+import lasso
 
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy as _
 from django.db import models
-from django.contrib.auth.models import User
+from django.conf import settings
 try:
     import ldap
 except ImportError:
     ldap = None
 
-from authentic2.attribute_aggregator.signals import any_attributes_call, \
-    listed_attributes_call, listed_attributes_with_source_call
-from authentic2.attribute_aggregator.mapping import ATTRIBUTE_MAPPING, \
-    ATTRIBUTE_NAMESPACES
-from authentic2.attribute_aggregator.core import convert_from_string, \
-    get_def_name_from_name_and_ns_of_attribute, iso8601_to_datetime, \
-    get_def_name_from_oid, get_def_name_from_alias, \
-    is_alias_of_definition, is_oid_of_definition
+from .mapping_loader import ATTRIBUTE_MAPPING, ATTRIBUTE_NAMESPACES
+from .core import (convert_from_string,
+    get_def_name_from_name_and_ns_of_attribute, iso8601_to_datetime,
+    get_def_name_from_oid, get_def_name_from_alias,
+    is_alias_of_definition, is_oid_of_definition)
 
 
-logger = logging.getLogger('attribute_aggregator')
+logger = logging.getLogger(__name__)
 
+ATTRIBUTES = [(key, key) \
+    for key in sorted(ATTRIBUTE_MAPPING.iterkeys())]
 
 ATTRIBUTES_NS = [('Default', 'Default')] \
     + [(ns, ns) for ns in ATTRIBUTE_NAMESPACES]
 
+ATTRIBUTE_VALUE_FORMATS = (
+        (lasso.SAML2_ATTRIBUTE_NAME_FORMAT_URI, 'SAMLv2 URI'),
+        (lasso.SAML2_ATTRIBUTE_NAME_FORMAT_BASIC, 'SAMLv2 BASIC'))
+
+
+class AttributeItem(models.Model):
+    attribute_name = models.CharField(
+        verbose_name = _("Attribute name"),
+        max_length = 100, choices = ATTRIBUTES,
+        default = ATTRIBUTES[0])
+    # ATTRIBUTE_VALUE_FORMATS[0] =>
+    #    (lasso.SAML2_ATTRIBUTE_NAME_FORMAT_BASIC, 'SAMLv2 BASIC')
+    output_name_format = models.CharField(
+        verbose_name = _("Output name format"),
+        max_length = 100,
+        choices = ATTRIBUTE_VALUE_FORMATS,
+        default = ATTRIBUTE_VALUE_FORMATS[0])
+    #ATTRIBUTES_NS[0] => ('Default', 'Default')
+    output_namespace = models.CharField(
+        verbose_name = _("Output namespace"),
+        max_length = 100,
+        choices = ATTRIBUTES_NS, default = ATTRIBUTES_NS[0])
+    required = models.BooleanField(
+        verbose_name = _("Required"),
+        default=False)
+    source = models.ForeignKey('attribute_aggregator.AttributeSource',
+        verbose_name = _("Attribute source"),
+        blank = True, null = True)
+
+    class Meta:
+        verbose_name = _('attribute list item')
+        verbose_name_plural = _('attribute list items')
+
+    def __unicode__(self):
+        s = self.attribute_name
+        attributes = []
+        attributes.append(u'output name fomat: %s' % self.output_name_format)
+        attributes.append(u'output namespace: %s' % self.output_namespace)
+        if self.required:
+            attributes.append(u'required')
+        if self.source:
+            attributes.append(u'source: %s' % self.source)
+        s += u' (%s)' % u', '.join(attributes)
+        return s
+
+    def __repr__(self):
+        return '<AttributeItem {0!r}>'.format(
+                self.__dict__)
+
+
+class AttributeList(models.Model):
+    name = models.CharField(
+        verbose_name = _("Name"),
+        max_length = 100, unique = True)
+    attributes = models.ManyToManyField(AttributeItem,
+        verbose_name = _("Attributes"),
+        related_name = "attributes of the list",
+        blank = True, null = True)
+
+    class Meta:
+        verbose_name = _('attribute list')
+        verbose_name_plural = _('attribute lists')
+
+    def __unicode__(self):
+        return self.name
+
+    def __repr__(self):
+        return '<AttributeList name:{0!r} attributes:[{1:r}]>'.format(
+                self.name, ', '.join(map(repr, self.attributes.all())))
+
+
 
 class AttributeSource(models.Model):
     name = models.CharField(
@@ -67,6 +139,10 @@ class AttributeSource(models.Model):
             pass
         return None
 
+    class Meta:
+        verbose_name = _('attribute source')
+        verbose_name_plural = _('attribute sources')
+
 
 def get_source_from_name(name):
     try:
@@ -112,28 +188,39 @@ if ldap:
             super(LdapSource, self).__init__(*args, **kwargs)
             self.namespace = "X500"
 
+        def get_url(self):
+            port = ''
+            if not (self.port == 389 or (self.port == 636 and self.ldaps)):
+                port = ':%s' % str(self.port)
+            return "ldap%s://%s%s" % (self.ldaps and 's' or '',
+                self.server, port)
+
+        class Meta:
+            verbose_name = _('ldap attribute source')
+            verbose_name_plural = _('ldap attribute sources')
+
+
 class UserAliasInSource(models.Model):
     name = models.CharField(
         verbose_name = _("Name"),
         max_length = 200)
     source = models.ForeignKey(AttributeSource,
-        verbose_name = _('Attribute Source'))
-    user = models.ForeignKey(User,
-        verbose_name = _("User"),
+        verbose_name = _('attribute source'))
+    user = models.ForeignKey(settings.AUTH_USER_MODEL,
+        verbose_name = _("user"),
         related_name='user_alias_in_source')
 
     class Meta:
-        verbose_name = _('alias in source')
-        verbose_name_plural = _('aliases in source')
+        verbose_name = _('user alias from source')
+        verbose_name_plural = _('user aliases from source')
         unique_together = ("name", "source")
 
     def __unicode__(self):
-        return "alias %s of user %s in %s" % (self.name, self.user,
+        return u"alias %s of user %s in %s" % (self.name, self.user,
             self.source)
 
 
 class AttributeData:
-
     def __init__(self, definition, values=None, source=None,
             expiration_date=None):
         '''
@@ -241,22 +328,40 @@ class AttributeData:
         else:
             return 0
 
+    def to_dic(self):
+        '''
+            a1 = dict()
+                a1['oid'] = definition_name
+            Or
+                a1['definition'] = definition_name
+                    definition may be the definition name like 'gn'
+                    or an alias like 'givenName'
+            a1['values'] = list_of_values
+        '''
+        attribute = dict()
+        attribute['definition'] = self.get_definition()
+        definition = self.get_full_definition()
+        if 'oid' in definition:
+            attribute['oid'] = definition['oid']
+        attribute['values'] = self.get_values()
+        return attribute
+
     def __unicode__(self):
-        s = "AttributeData"
+        s = u"AttributeData"
         values = self.get_values()
         if values:
-            s += " %s with values %s" % (self.get_definition(), values)
+            s += u" %s with values %s" % (self.get_definition(), values)
         source = self.get_source()
         if source:
-            s += " from %s" % str(source)
+            s += u" from %s" % str(source)
         if self.does_expire():
-            s += " (Expires on %s)" % self.does_expire()
+            s += u" (Expires on %s)" % self.does_expire()
         return s
 
 
 class UserAttributeProfile(models.Model):
-    user = models.OneToOneField(User, null=True, blank=True,
-        related_name='user_attribute_profile')
+    user = models.OneToOneField(settings.AUTH_USER_MODEL, null=True,
+            blank=True, related_name='user_attribute_profile')
     data = models.TextField(null=True, blank=True)
 
     def add_data(self, data):
@@ -291,6 +396,34 @@ class UserAttributeProfile(models.Model):
         except:
             return []
 
+    def get_all_data_to_dic(self):
+        '''
+            Dictionnary format:
+            attributes = dict()
+            data_from_source = list()
+            a1 = dict()
+                a1['oid'] = definition_name
+            Or
+                a1['definition'] = definition_name
+                    definition may be the definition name like 'gn'
+                    or an alias like 'givenName'
+            Or
+                a1['name'] = attribute_name_in_ns
+                a1['namespace'] = ns_name
+            a1['values'] = list_of_values
+            data_from_source.append(a1)
+            ...
+            data_from_source.append(a2)
+            attributes[source_name] = data_from_source
+        '''
+        all_data = self.get_all_data()
+        all_data_to_dic = dict()
+        if all_data:
+            for data in all_data:
+                all_data_to_dic.setdefault(data.get_source().name,
+                    []).append(data.to_dic())
+        return all_data_to_dic
+
     def get_data_of_definition(self, definition, in_list=None):
         '''
             definition can be given by its name, an alias or an oid
@@ -368,8 +501,7 @@ class UserAttributeProfile(models.Model):
             Else, definition is searched by 'name' and 'namespece' keys.
         '''
         if not dictionnary:
-            logger.error('load_by_dic: \
-                Missing profile or dictionnary')
+            logger.info('load_by_dic: empty dictionnary provided')
             return -1
         for source_name in dictionnary:
             logger.debug('load_by_dic: loading from source with name: %s' \
@@ -456,9 +588,12 @@ class UserAttributeProfile(models.Model):
                 logger.info('load_greedy: \
                     attributes_call connected to function %s' % \
                     attrs[0].__name__)
-                logger.info('load_greedy: \
-                    attributes provided are %s' %str(attrs[1]))
-                self.load_by_dic(attrs[1])
+                if attrs[1]:
+                    logger.info('load_greedy: \
+                        attributes provided are %s' %str(attrs[1]))
+                    self.load_by_dic(attrs[1])
+                else:
+                    logger.info('load_greedy: no attributes provided')
 
     def load_listed_attributes(self, definitions):
         '''
@@ -486,14 +621,18 @@ class UserAttributeProfile(models.Model):
                     logger.info('load_listed_attributes: \
                         attributes_call connected to function %s' % \
                         attrs[0].__name__)
-                    logger.info('load_listed_attributes: \
-                        attributes provided are %s' %str(attrs[1]))
-                    self.load_by_dic(attrs[1])
+                    if attrs[1]:
+                        logger.info('load_listed_attributes: \
+                            attributes provided are %s' %str(attrs[1]))
+                        self.load_by_dic(attrs[1])
+                    else:
+                        logger.info('load_listed_attributes: \
+                            no attributes provided')
             else:
                 logger.info('load_listed_attributes: no definitions \
                     of attributes to load with %s' % str(definitions))
 
-    def load_listed_attributes_with_source(self, definitions, source):
+    def load_listed_attributes_with_source(self, definitions, source, auth_source=False):
         if not source:
             return
         if self.user:
@@ -510,22 +649,41 @@ class UserAttributeProfile(models.Model):
                         if df:
                             defs.append(df)
             if defs:
-                logger.info('load_listed_attributes: \
+                logger.info('load_listed_attributes_with_source: \
                     attributes required are %s from %s' % (defs, source))
                 attributes_provided = \
                     listed_attributes_with_source_call.send(sender=None,
-                        user=self.user, definitions=defs, source=source)
+                        user=self.user, definitions=defs, source=source, auth_source=auth_source)
                 for attrs in attributes_provided:
-                    logger.info('load_listed_attributes: \
+                    logger.info('load_listed_attributes_with_source: \
                         attributes_call connected to function %s' % \
                         attrs[0].__name__)
-                    logger.info('load_listed_attributes: \
-                        attributes provided are %s' %str(attrs[1]))
-                    self.load_by_dic(attrs[1])
+                    if attrs[1]:
+                        logger.info('load_listed_attributes_with_source: \
+                            attributes provided are %s' %str(attrs[1]))
+                        self.load_by_dic(attrs[1])
+                    else:
+                        logger.info('load_listed_attributes_with_source: \
+                            no attributes provided')
             else:
                 logger.info('load_listed_attributes: no definitions \
                     of attributes to load with %s' % str(definitions))
 
+    def process(self, context=None):
+        all_data = self.get_all_data_to_dic()
+        connecteds = \
+            add_attributes.send(sender=self.__class__, instance=self,
+                user=self.user, attributes=all_data, context=context)
+        logger.info('process_with_scripts: '
+            'signal add_attributes sent')
+        for connected in connecteds:
+            logger.info('process_with_scripts: add_attributes '
+                'connected to function %s' % connected[0].__name__)
+            if connected[1]:
+                logger.info('process_with_scripts: attributes provided are '
+                    '%s' % str(connected[1]))
+                self.load_by_dic({'PROCESSING': connected[1]})
+
     def cleanup(self):
         l = self.get_all_data()
         if not l:
@@ -536,15 +694,22 @@ class UserAttributeProfile(models.Model):
         self.save()
 
     def __unicode__(self):
-        s = None
+        s = u""
         if self.user:
-            s = "Profile of %s" % self.user
+            s = u"Profile of %s" % self.user
         else:
-            s = "Anonymous profile"
+            s = u"Anonymous profile"
         if not self.get_all_data():
-            s += " is empty."
+            s += u" is empty."
             return s
-        s += " that contains:"
+        s += u" that contains:"
         for d in self.get_all_data():
-            s = s + "\n\t" + d.__unicode__()
+            s = s + u"\n\t" + d.__unicode__()
         return s
+
+    class Meta:
+        verbose_name = _('user attribute profile')
+        verbose_name_plural = _('user attribute profiles')
+
+from authentic2.attribute_aggregator.signals import any_attributes_call, \
+    listed_attributes_call, listed_attributes_with_source_call, add_attributes
