@@ -1,4 +1,6 @@
-from idp.views import accumulate_from_backends
+from django.conf import settings
+
+from . import utils
 
 class UserFederations(object):
     '''Provide access to all federations of the current user'''
@@ -13,7 +15,7 @@ class UserFederations(object):
             except ValueError:
                 pass
             else:
-                links = accumulate_from_backends(self.request, 'links')
+                links = utils.accumulate_from_backends(self.request, 'links')
                 for provider, link in links:
                     if provider.id != provider_id:
                         continue
@@ -22,5 +24,8 @@ class UserFederations(object):
             return d
         return super(UserFederations, self).__getattr__(name)
 
-def federations_processor(request):
-    return {'federations': UserFederations(request) }
+def a2_processor(request):
+    variables = {}
+    variables.update(getattr(settings, 'TEMPLATE_VARS', {}))
+    variables['federations'] = UserFederations(request)
+    return variables
