@@ -345,8 +345,7 @@ class SAMLAttribute(models.Model):
             verbose_name=_('friendly name'),
             blank=True)
     attribute_name = models.CharField(max_length=64,
-            verbose_name=_('attribute name'),
-            choices=(('', 'None'),))
+            verbose_name=_('attribute name'))
 
     def clean(self):
         super(SAMLAttribute, self).clean()
@@ -513,9 +512,14 @@ class LibertyServiceProvider(models.Model):
              related_name = "attribute_policy",
             verbose_name=_("attribute policy"), null=True, blank=True)
 
+    objects = managers.GetByLibertyProviderManager()
+
     def get_policy(self):
         return get_all_custom_or_default(self, 'policy')
 
+    def natural_key(self):
+        return (self.liberty_provider.slug,)
+
     def __unicode__(self):
         return unicode(self.liberty_provider)
 
@@ -546,6 +550,8 @@ class LibertyIdentityProvider(models.Model):
             related_name="authorization_policy",
             verbose_name=_('authorization identity providers policy'), blank=True, null=True)
 
+    objects = managers.GetByLibertyProviderManager()
+
     # TODO: add clean method which checks that the LassoProvider we can create
     # with the metadata file support the IDP role
     # i.e. provider.roles & lasso.PROVIDER_ROLE_IDP != 0
@@ -553,6 +559,9 @@ class LibertyIdentityProvider(models.Model):
     def __unicode__(self):
         return unicode(self.liberty_provider)
 
+    def natural_key(self):
+        return (self.liberty_provider.slug,)
+
     class Meta:
         verbose_name = _('liberty identity provider')
         verbose_name_plural = _('liberty identity providers')
@@ -662,6 +671,19 @@ class LibertyFederation(models.Model):
             kwargs.update(nameid2kwargs(name_id))
         models.Model.__init__(self, *args, **kwargs)
 
+    def natural_key(self):
+        key = self.user.natural_key()
+        if self.sp:
+            key += self.sp.natural_key()
+        else:
+            key += (None,)
+        if self.idp:
+            key += self.idp.natural_key()
+        else:
+            key += (None,)
+        return key
+
+
     def is_unique(self, for_format=True):
         '''Return whether a federation already exist for this user and this provider.
 
