@@ -5,6 +5,7 @@ import glob
 import re
 
 from django.core.exceptions import ImproperlyConfigured
+from django.conf import global_settings
 
 from . import plugins
 
@@ -150,6 +151,14 @@ TEMPLATE_VARS = {}
 if os.environ.get('STATICFILES_DIRS'):
     STATICFILES_DIRS = tuple(os.environ['STATICFILES_DIRS'].split(':')) + STATICFILES_DIRS
 
+LOCALE_PATHS = (
+        os.path.join(VAR_DIR, 'locale'),
+        os.path.join(BASE_DIR, 'locale'),
+)
+if os.environ.get('LOCALE_PATHS'):
+    LOCALE_PATHS = tuple(os.environ['LOCALE_PATHS'].split(':')) + LOCALE_PATHS
+
+
 
 INSTALLED_APPS = (
     'django.contrib.staticfiles',
@@ -164,6 +173,8 @@ INSTALLED_APPS = (
     'admin_tools.dashboard',
     'django.contrib.admin',
     'registration',
+    'django_select2',
+    'django_tables2',
     'authentic2.nonce',
     'authentic2.saml',
     'authentic2.idp',
@@ -171,6 +182,7 @@ INSTALLED_APPS = (
     'authentic2.auth2_auth',
     'authentic2.attribute_aggregator',
     'authentic2.disco_service',
+    'authentic2.manager',
     'authentic2',
 )
 
@@ -309,8 +321,19 @@ SAML_METADATA_AUTOLOAD = os.environ.get('SAML_METADATA_AUTOLOAD', 'none')
 
 PUSH_PROFILE_UPDATES = 'PUSH_PROFILE_UPDATES' in os.environ
 
+
 if 'PASSWORD_HASHERS' in os.environ:
     PASSWORD_HASHERS = os.environ['PASSWORD_HASHERS'].split(':')
+else:
+    PASSWORD_HASHERS = global_settings.PASSWORD_HASHERS
+    PASSWORD_HASHERS += (
+            'authentic2.hashers.Drupal7PasswordHasher',
+            'authentic2.hashers.SHA256PasswordHasher',
+            'authentic2.hashers.SSHA1PasswordHasher',
+            'authentic2.hashers.SMD5PasswordHasher',
+            'authentic2.hashers.SHA1OLDAPPasswordHasher',
+            'authentic2.hashers.MD5OLDAPPasswordHasher',
+    )
 
 ##################################
 # LDAP Configuration
@@ -369,6 +392,7 @@ SOUTH_TESTS_MIGRATE = False
 ############################
 # Registration
 ############################
+A2_ACCEPT_EMAIL_AUTHENTICATION = to_boolean('A2_ACCEPT_EMAIL_AUTHENTICATION')
 A2_CAN_RESET_PASSWORD = to_boolean('A2_CAN_RESET_PASSWORD')
 A2_REGISTRATION_CAN_DELETE_ACCOUNT = to_boolean('A2_REGISTRATION_CAN_DELETE_ACCOUNT')
 A2_REGISTRATION_EMAIL_IS_UNIQUE = to_boolean('A2_REGISTRATION_EMAIL_IS_UNIQUE', default=False)
@@ -566,5 +590,3 @@ if IDP_OPENID:
 if LDAP_AUTH_SETTINGS:
     AUTHENTICATION_BACKENDS = ('authentic2.backends.LDAPBackend',) + AUTHENTICATION_BACKENDS
 
-if DEBUG:
-    print 'Debugging mode is active'
