@@ -14,15 +14,11 @@ class HashersTests(TestCase):
     def test_openldap_hashers(self):
         VECTORS = map(str.split, '''\
 coin {SHA}NHj+acfc68FPYrMipEBZ3t8ABGY=
+250523 {SHA}4zuJhPW1w0upqG7beAlxDcvtBj0=
 coin {SSHA}zLPxfZ3RSNkIwVdHWEyB4Tpr6fT9LiVX
 coin {SMD5}+x9QkU2T/wlPp6NK3bfYYxPYwaE=
 coin {MD5}lqlRm4/d0X6MxLugQI///Q=='''.splitlines())
-        with self.settings(PASSWORD_HASHERS=(
-            'authentic2.hashers.SSHA1PasswordHasher',
-            'authentic2.hashers.SMD5PasswordHasher',
-            'authentic2.hashers.SHA1OLDAPPasswordHasher',
-            'authentic2.hashers.MD5OLDAPPasswordHasher')):
-            for password, oldap_hash in VECTORS:
-                dj_hash = hashers.olap_password_to_dj(oldap_hash)
-                self.assertTrue(check_password(password, dj_hash))
+        for password, oldap_hash in VECTORS:
+            dj_hash = hashers.olap_password_to_dj(oldap_hash)
+            self.assertTrue(check_password(password, dj_hash))
 
