@@ -17,20 +17,17 @@ from django.contrib.auth import SESSION_KEY
 from django import http, shortcuts
 from django.core import mail, signing
 from django.core.urlresolvers import reverse
-from django.contrib.sites.models import get_current_site
 from django.contrib import messages
 from django.utils.translation import ugettext as _
 from django.utils.http import urlencode
 from django.contrib.auth import logout as auth_logout
 from django.contrib.auth import REDIRECT_FIELD_NAME
-from django.contrib.auth.models import SiteProfileNotAvailable
 from django.http import (HttpResponseRedirect, HttpResponseForbidden,
     HttpResponse)
-from django.core.exceptions import ObjectDoesNotExist
 from django.views.decorators.csrf import csrf_protect
-from django.contrib.sites.models import Site, RequestSite
 from django.views.decorators.cache import never_cache
 from django.contrib.auth.decorators import login_required
+from django.db.models.fields import FieldDoesNotExist
 
 
 # FIXME: this decorator has nothing to do with an idp, should be moved in the
@@ -38,7 +35,8 @@ from django.contrib.auth.decorators import login_required
 # FIXME: this constant should be moved in the a2 package
 
 
-from . import utils, app_settings, forms, compat, decorators, constants
+from . import (utils, app_settings, forms, compat, decorators,
+    constants, models)
 
 
 logger = logging.getLogger(__name__)
@@ -141,7 +139,7 @@ class EmailChangeView(FormView):
     template_name = 'profiles/email_change.html'
     subject_template = 'profiles/email_change_subject.txt'
     body_template = 'profiles/email_change_body.txt'
-    success_url = '../..'
+    success_url = '..'
 
     def get_form_kwargs(self):
         kwargs = super(EmailChangeView, self).get_form_kwargs()
@@ -152,10 +150,8 @@ class EmailChangeView(FormView):
 
     def form_valid(self, form):
         email = form.cleaned_data['email']
-        site = get_current_site(self.request)
         token = signing.dumps({
             'email': email,
-            'site_pk': site.pk,
             'user_pk': self.request.user.pk,
         })
         link = '{0}?token={1}'.format(
@@ -163,15 +159,14 @@ class EmailChangeView(FormView):
                 token)
         link = self.request.build_absolute_uri(link)
         ctx = {'email': email,
-               'site': site,
                'user': self.request.user,
-               'link': link
+               'link': link,
+               'domain': self.request.get_host(),
         }
         subject = render_to_string(self.subject_template, ctx).strip()
         body = render_to_string(self.body_template, ctx)
 
-        mail.EmailMessage(subject=subject,
-                body=body, to=[email]).send()
+        mail.EmailMessage(subject=subject, body=body, to=[email]).send()
         messages.info(self.request,
                 _('Your request for changing your email '
                   'is received. An email of validation '
@@ -189,10 +184,6 @@ class EmailChangeVerifyView(TemplateView):
                 token = signing.loads(request.GET['token'], max_age=7200)
                 user_pk = token['user_pk']
                 email = token['email']
-                site_pk = token['site_pk']
-                site = get_current_site(request)
-                if site.pk != site_pk:
-                    raise ValueError
                 user = User.objects.get(pk=user_pk)
                 user.email = email
                 user.save()
@@ -210,7 +201,10 @@ class EmailChangeVerifyView(TemplateView):
             except User.DoesNotExist:
                 messages.error(request, _('your request for changing your email is for '
                     'an unknown user, try again'))
-        return shortcuts.redirect(settings.LOGIN_REDIRECT_URL)
+            else:
+                return shortcuts.redirect('account_management')
+        return shortcuts.redirect('email-change')
+
 
 email_change_verify = EmailChangeVerifyView.as_view()
 
@@ -283,16 +277,9 @@ def login(request, template_name='auth/login.html',
 
     request.session.set_test_cookie()
 
-    if Site._meta.installed:
-        current_site = Site.objects.get_current()
-    else:
-        current_site = RequestSite(request)
-
     return render_to_response(template_name, {
         'methods': rendered_forms,
         redirect_field_name: redirect_to,
-        'site': current_site,
-        'site_name': current_site.name,
     }, context_instance=RequestContext(request))
 
 
@@ -316,51 +303,75 @@ def _homepage(request):
     return render_to_response('idp/homepage.html',
        tpl_parameters, RequestContext(request))
 
-@decorators.prevent_access_to_transient_users
-def profile(request):
+class ProfileView(TemplateView):
+    template_name = 'idp/account_management.html'
 
-    frontends = utils.get_backends('AUTH_FRONTENDS')
+    def get_context_data(self, **kwargs):
+        ctx = super(ProfileView, self).get_context_data(**kwargs)
+        frontends = utils.get_backends('AUTH_FRONTENDS')
+        request = self.request
 
-    if request.method == "POST":
-        for frontend in frontends:
-            if not frontend.enabled():
-                continue
-            if 'submit-%s' % frontend.id() in request.POST:
-                form = frontend.form()(data=request.POST)
-                if form.is_valid():
-                    if request.session.test_cookie_worked():
-                        request.session.delete_test_cookie()
-                    return frontend.post(request, form, None, '/profile')
-    # User attributes management
-    profile = []
-    try:
-        for field_name in getattr(request.user, 'USER_PROFILE', []):
-            if isinstance(field_name, tuple):
-                field_name, title = field_name
-            elif isinstance(field_name, str):
-                title = request.user._meta.get_field(field_name).verbose_name
+        if request.method == "POST":
+            for frontend in frontends:
+                if not frontend.enabled():
+                    continue
+                if 'submit-%s' % frontend.id() in request.POST:
+                    form = frontend.form()(data=request.POST)
+                    if form.is_valid():
+                        if request.session.test_cookie_worked():
+                            request.session.delete_test_cookie()
+                        return frontend.post(request, form, None, '/profile')
+        # User attributes management
+        profile = []
+        field_names = app_settings.A2_PROFILE_FIELDS
+        if not field_names:
+            field_names = list(app_settings.A2_REGISTRATION_FIELDS)
+            for field_name in getattr(request.user, 'USER_PROFILE', []):
+                if field_name not in field_names:
+                    field_names.append(field_name)
+            qs = models.Attribute.objects.filter(user_visible=True)
+            qs = qs.values_list('name', flat=True)
+            for field_name in qs:
+                if field_name not in field_names:
+                    field_names.append(field_name)
+        for field_name in field_names:
+            title = None
+            if isinstance(field_name, (list, tuple)):
+                if len(field_name) > 1:
+                    title = field_name[1]
+                field_name = field_name[0]
+            try:
+                field = request.user._meta.get_field(field_name)
+            except FieldDoesNotExist:
+                qs = models.AttributeValue.objects.with_owner(request.user)
+                qs = qs.filter( attribute__name=field_name, attribute__user_visible=True)
+                qs = qs.select_related()
+                value = [at_value.to_python() for at_value in qs]
+                value = filter(None, value)
+                if qs and not title:
+                    title = unicode(qs[0].attribute)
             else:
-                raise TypeError('USER_PROFILE must contain string or tuple')
-            value = getattr(request.user, field_name, None)
+                if not title:
+                    title = field.verbose_name
+                value = getattr(self.request.user, field_name, None)
             if not value:
                 continue
             if callable(value):
                 value = value()
-            if not isinstance(value, basestring) and hasattr(value, '__iter__'):
-                profile.append((title, map(unicode, value)))
-            else:
-                profile.append((title, [unicode(value)]))
-    except (SiteProfileNotAvailable, ObjectDoesNotExist):
-        pass
-    # Credentials management
-    blocks = [ frontend.profile(request) for frontend in frontends \
-            if hasattr(frontend, 'profile') ]
-    return render_to_response('idp/account_management.html', {
-        'frontends_block': blocks,
-        'profile': profile,
-        'allow_account_deletion': app_settings.A2_REGISTRATION_CAN_DELETE_ACCOUNT,
-        },
-        RequestContext(request))
+            if not isinstance(value, (list, tuple)):
+                value = (value,)
+            profile.append((title, map(unicode, value)))
+        # Credentials management
+        blocks = [ frontend.profile(request) for frontend in frontends \
+                if hasattr(frontend, 'profile') ]
+        ctx.update({
+            'frontends_block': blocks,
+            'profile': profile,
+            'allow_account_deletion': app_settings.A2_REGISTRATION_CAN_DELETE_ACCOUNT,
+        })
+        return ctx
+
+profile = decorators.prevent_access_to_transient_users(ProfileView.as_view())
 
 def logout_list(request):
     '''Return logout links from idp backends'''
